@@ -8,7 +8,7 @@ Generic library for managing signed authentication tokens.
 """
 
 __ver_major__ = 0
-__ver_minor__ = 2
+__ver_minor__ = 3
 __ver_patch__ = 0
 __ver_sub__ = ""
 __ver_tuple__ = (__ver_major__, __ver_minor__, __ver_patch__, __ver_sub__)
@@ -21,8 +21,10 @@ import time
 import json
 import hmac
 import hashlib
+import warnings
 from binascii import hexlify
 
+from tokenlib import errors
 from tokenlib.utils import (strings_differ, HKDF,
                             encode_token_bytes, decode_token_bytes)
 
@@ -35,7 +37,12 @@ logger = logging.getLogger('tokenlib')
 #  functions will have a consistent view of e.g. the default secret.
 DEFAULT_SECRET = os.urandom(32)
 DEFAULT_TIMEOUT = 5 * 60
-DEFAULT_HASHMOD = hashlib.sha1
+DEFAULT_HASHMOD = hashlib.sha256
+
+
+#  Unique info strings for mixing into HKDF.
+HKDF_INFO_SIGNING = b"services.mozilla.com/tokenlib/v1/signing"
+HKDF_INFO_DERIVE = b"services.mozilla.com/tokenlib/v1/derive/"
 
 
 class TokenManager(object):
@@ -49,7 +56,7 @@ class TokenManager(object):
     The TokenManager must be initialized with a "master secret" which is used
     to crytographically secure the tokens.  Each token consists of a JSON
     object with an appended HMAC signature.  Tokens also have a corresponding
-    "token secret" generated using HKDF, which can be given to clients for
+    "derived secret" generated using HKDF, which can be given to clients for
     use in signature-based authentication schemes.
 
     The constructor takes the following arguments:
@@ -57,10 +64,10 @@ class TokenManager(object):
        * secret:  string key used for signing the token;
                   if not specified then a random bytestring is used.
 
-       * timeout: the time after which a token will expire.
+       * timeout: the time after which a token will expire, in seconds.
 
        * hashmod:  the hashing module to use for various HMAC operations;
-                   if not specified then hashlib.sha1 will be used.
+                   if not specified then hashlib.sha256 will be used
 
     """
 
@@ -78,9 +85,13 @@ class TokenManager(object):
         self.secret = secret
         self.timeout = timeout
         self.hashmod = hashmod
-        self.hashmod_digest_size = hashmod().digest_size
-        self._sig_secret = HKDF(self.secret, salt=None, info=b"SIGNING",
-                                size=self.hashmod_digest_size)
+        hashobj = hashmod()
+        self.hashmod_name = hashobj.name
+        self.hashmod_digest_size = hashobj.digest_size
+        self._sig_secret = HKDF(self.secret, salt=None,
+                                info=HKDF_INFO_SIGNING,
+                                size=self.hashmod_digest_size,
+                                hashmod=self.hashmod)
 
     def make_token(self, data):
         """Generate a new token embedding the given dict of data.
@@ -109,8 +120,8 @@ class TokenManager(object):
         # Parse the payload and signature from the token.
         try:
             decoded_token = decode_token_bytes(token)
-        except TypeError as e:
-            raise ValueError(str(e))  # pragma: nocover
+        except (TypeError, ValueError) as e:
+            raise errors.MalformedTokenError(str(e))
         payload = decoded_token[:-self.hashmod_digest_size]
         sig = decoded_token[-self.hashmod_digest_size:]
         # Carefully check the signature.
@@ -118,34 +129,46 @@ class TokenManager(object):
         # Read the docstring of strings_differ for more details.
         expected_sig = self._get_signature(payload)
         if strings_differ(sig, expected_sig):
-            raise ValueError("token has invalid signature")
+            raise errors.InvalidSignatureError()
         # Only decode *after* we've confirmed the signature.
-        data = json.loads(payload.decode("utf8"))
+        # This should never fail, but well, you can't be too careful.
+        try:
+            data = json.loads(payload.decode("utf8"))
+        except ValueError as e:  # pragma: nocover
+            raise errors.MalformedTokenError(str(e))
         # Check whether it has expired.
         if now is None:
             now = time.time()
         if data["expires"] <= now:
-            raise ValueError("token has expired")
+            raise errors.ExpiredTokenError()
         return data
 
     def get_token_secret(self, token):
-        """Get the secret key associated with the given token.
+        """Get the derived secret key associated with the given token.
+
+        A per-token secret key is calculated by deriving it from the master
+        secret with HKDF.
+
+        DEPRECATED: use the get_derived_secret() method instead.
+        """
+        warnings.warn("get_token_secret is deprecated; use get_derived_secret",
+                      category=DeprecationWarning)
+        return self.get_derived_secret(token)
+
+    def get_derived_secret(self, token):
+        """Get the derived secret key associated with the given token.
 
         A per-token secret key is calculated by deriving it from the master
         secret with HKDF.
         """
-        # XXX: Having to parse the salt back out of the token is yuck.
-        # But I like having get_token_secret() as an independent method.
-        # We should consider modifying token format to make this easier.
-        # e.g. by having token = b64encode(data):salt:signature.
         try:
             payload = decode_token_bytes(token)[:-self.hashmod_digest_size]
             salt = json.loads(payload.decode("utf8"))["salt"].encode("ascii")
-        except (TypeError, KeyError) as e:
-            raise ValueError(str(e))
-        info = token.encode("ascii")
-        size = self.hashmod_digest_size
-        secret = HKDF(self.secret, salt=salt, info=info, size=size)
+        except (TypeError, KeyError, ValueError) as e:
+            raise errors.MalformedTokenError(str(e))
+        info = HKDF_INFO_DERIVE + token.encode("ascii")
+        secret = HKDF(self.secret, salt=salt, info=info,
+                      size=self.hashmod_digest_size, hashmod=self.hashmod)
         return encode_token_bytes(secret)
 
     def _get_signature(self, value):
@@ -164,5 +187,13 @@ def parse_token(token, now=None, **kwds):
 
 
 def get_token_secret(token, **kwds):
-    """Convenience function to get the secret key for a given token."""
+    """Convenience function to get the derived secret key for a given token.
+
+    DEPRECATED: use get_derived_secret() instead.
+    """
     return TokenManager(**kwds).get_token_secret(token)
+
+
+def get_derived_secret(token, **kwds):
+    """Convenience function to get the derived secret key for a given token."""
+    return TokenManager(**kwds).get_derived_secret(token)
